﻿using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using Microsoft.Office.Interop.Outlook;
using Palmmedia.GoogleSync.Common;

namespace Palmmedia.GoogleSync
{
    /// <summary>
    /// Monitors the default calendar for new events.
    /// </summary>
    internal class CalendarMonitor
    {
        /// <summary>
        /// Reference to the watched folder.
        /// </summary>
        private MAPIFolder folder;

        /// <summary>
        /// Reference to the watched calendar items.
        /// </summary>
        private List<Items> calendarItems;

        /// <summary>
        /// Initializes a new instance of the <see cref="CalendarMonitor"/> class.
        /// </summary>
        /// <param name="explorer">The explorer.</param>
        public CalendarMonitor(Explorer explorer)
        {
            this.calendarItems = new List<Items>();

            NameSpace session = explorer.Session;
            try
            {
                this.HookupDefaultCalendarEvents(session);
            }
            finally
            {
                Marshal.ReleaseComObject(session);
                session = null;
            }
        }

        /// <summary>
        /// Occurs when appointment is added to the default calendar.
        /// </summary>
        public event EventHandler<EventArgs<AppointmentItem>> AppointmentAdded;

        /// <summary>
        /// Start monitoring calendar events.
        /// </summary>
        /// <param name="session">The session.</param>
        private void HookupDefaultCalendarEvents(NameSpace session)
        {
            MAPIFolder folder = session.GetDefaultFolder(OlDefaultFolders.olFolderCalendar);
            if (folder != null)
            {
                try
                {
                    Items items = folder.Items;

                    // Keep reference to folder and items to prevent loosing events
                    this.folder = folder;
                    this.calendarItems.Add(items);

                    items.ItemAdd += new ItemsEvents_ItemAddEventHandler(this.OnAppointmentAdded);
                }
                finally
                {
                    Marshal.ReleaseComObject(folder);
                    folder = null;
                }
            }
        }

        /// <summary>
        /// Called when an appointment is added.
        /// </summary>
        /// <param name="item">An item.</param>
        private void OnAppointmentAdded(object item)
        {
            AppointmentItem appointment = item as AppointmentItem;
            if (appointment != null)
            {
                try
                {
                    if (this.AppointmentAdded != null)
                    {
                        this.AppointmentAdded(this, new EventArgs<AppointmentItem>(appointment));
                    }
                }
                finally
                {
                    Marshal.ReleaseComObject(appointment);
                    appointment = null;
                }
            }
        }
    }
}
