﻿using System.Windows.Forms;
using GoogleSync.Properties;

namespace Palmmedia.GoogleSync
{
    /// <summary>
    /// Dialog where credentials and options can be set.
    /// </summary>
    public partial class Credentials : Form
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="Credentials"/> class.
        /// </summary>
        public Credentials()
        {
            InitializeComponent();

            this.saveCredentialsCheckBox.Checked = Settings.Default.SaveCredentials;

            this.Username = Settings.Default.Username;
            this.Password = Settings.Default.Password;
            this.DeleteExistingContacts = Settings.Default.DeleteExistingContacts;
            this.CreateEvents = Settings.Default.CreateEvents;
        }

        #region Properties

        /// <summary>
        /// Gets the Google username.
        /// </summary>
        public string Username
        {
            get
            {
                return this.userNameTextBox.Text;
            }

            private set
            {
                this.userNameTextBox.Text = value;
            }
        }

        /// <summary>
        /// Gets the Google password.
        /// </summary>
        public string Password
        {
            get
            {
                return this.passwordTextBox.Text;
            }

            private set
            {
                this.passwordTextBox.Text = value;
            }
        }

        /// <summary>
        /// Gets a value indicating whether all existing contacts should be deleted.
        /// </summary>
        public bool DeleteExistingContacts
        {
            get
            {
                return this.deleteExistingContactsCheckBox.Checked;
            }

            private set
            {
                this.deleteExistingContactsCheckBox.Checked = value;
            }
        }

        /// <summary>
        /// Gets a value indicating whether calendar events are created.
        /// </summary>
        public bool CreateEvents
        {
            get
            {
                return this.createEventsCheckBox.Checked;
            }

            private set
            {
                this.createEventsCheckBox.Checked = value;
            }
        }

        #endregion

        /// <summary>
        /// Handles the Click event of the syncButton control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void syncButton_Click(object sender, System.EventArgs e)
        {
            if (this.saveCredentialsCheckBox.Checked)
            {
                Settings.Default.Username = this.Username;
                Settings.Default.Password = this.Password;
            }
            else
            {
                Settings.Default.Username = string.Empty;
                Settings.Default.Password = string.Empty;
            }

            Settings.Default.SaveCredentials = this.saveCredentialsCheckBox.Checked;
            Settings.Default.DeleteExistingContacts = this.DeleteExistingContacts;
            Settings.Default.CreateEvents = this.CreateEvents;
            Settings.Default.Save();

            this.DialogResult = System.Windows.Forms.DialogResult.OK;
            this.Close();
        }
    }
}
