﻿using System;
using System.Windows.Forms;
using Palmmedia.GoogleSync.MessageHandling;
using Palmmedia.GoogleSync.Synchronization;
using Office = Microsoft.Office.Core;

namespace Palmmedia.GoogleSync
{
    public partial class ThisAddIn
    {
        /// <summary>
        /// The button to sync with Google.
        /// </summary>
        private Office.CommandBarButton googleSyncCommandBarButton;

        /// <summary>
        /// Handles the Startup event of the ThisAddIn control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void ThisAddIn_Startup(object sender, System.EventArgs e)
        {
            try
            {
                // TODO: This may not work in english version of Outlook:
                var mainMenuBar = this.Application.ActiveExplorer().CommandBars["Standard"];

                this.googleSyncCommandBarButton = (Office.CommandBarButton)mainMenuBar.Controls.Add(Office.MsoControlType.msoControlButton, this.missing, this.missing, mainMenuBar.Controls.Count, true);
                this.googleSyncCommandBarButton.Style = Office.MsoButtonStyle.msoButtonIcon;
                this.googleSyncCommandBarButton.FaceId = 1977;
                this.googleSyncCommandBarButton.TooltipText = "Synchronize with Google";
                this.googleSyncCommandBarButton.Click += new Office._CommandBarButtonEvents_ClickEventHandler(this.googleSyncCommandBarButton_Click);
            }
            catch (Exception ex)
            {
                MessageBox.Show("Error during creating sync button: " + ex.Message);
            }
        }

        /// <summary>
        /// Starts synchronization process.
        /// </summary>
        /// <param name="ctrl">The button control.</param>
        /// <param name="cancel">If set to <c>true</c> cancel.</param>
        private void googleSyncCommandBarButton_Click(Office.CommandBarButton ctrl, ref bool cancel)
        {
            using (var credentials = new Credentials())
            {
                credentials.ShowDialog();

                if (credentials.DialogResult == DialogResult.OK)
                {
                    if (string.IsNullOrWhiteSpace(credentials.Username) || string.IsNullOrWhiteSpace(credentials.Password))
                    {
                        MessageBox.Show("You must enter a username and password.");
                    }
                    else
                    {
                        var settings = new GoogleSync.Synchronization.Settings()
                        {
                            Username = credentials.Username,
                            Password = credentials.Password,
                            DeleteExistingContacts = credentials.DeleteExistingContacts,
                            CreateEvents = credentials.CreateEvents
                        };

                        this.googleSyncCommandBarButton.Enabled = false;
                        this.googleSyncCommandBarButton.TooltipText = "Currently synchronizing. Please wait.";

                        var synchronizer = new Synchronizer(this.Application, new FormsMessageHandler());

                        EventHandler<EventArgs> finishedEventHandler = null;
                        finishedEventHandler = new EventHandler<EventArgs>((s, e) =>
                        {
                            this.googleSyncCommandBarButton.Enabled = true;
                            this.googleSyncCommandBarButton.TooltipText = "Synchronize with Google";
                            synchronizer.Finished -= finishedEventHandler;
                        });
                        synchronizer.Finished += finishedEventHandler;

                        synchronizer.Synchronize(settings);
                    }
                }
            }
        }

        /// <summary>
        /// Handles the Shutdown event of the ThisAddIn control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void ThisAddIn_Shutdown(object sender, System.EventArgs e)
        {
        }

        #region VSTO generated code

        /// <summary>
        /// Required method for Designer support - do not modify
        /// the contents of this method with the code editor.
        /// </summary>
        private void InternalStartup()
        {
            this.Startup += new System.EventHandler(ThisAddIn_Startup);
            this.Shutdown += new System.EventHandler(ThisAddIn_Shutdown);
        }

        #endregion
    }
}
